@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011 Aurimas Cernius
+ * Copyright (C) 2011-2012 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -69,11 +69,19 @@
 
 namespace gnote {
 
+  const int ActionManager::APP_ACTION_NEW = 1;
+  const int ActionManager::APP_ACTION_MANAGE = 2;
+  const int ActionManager::APP_ACTION_HELP = 3;
+  const int ActionManager::APP_ACTION_LAST = 4;
+
+
   ActionManager::ActionManager()
     : m_ui(Gtk::UIManager::create())
     , m_main_window_actions(Gtk::ActionGroup::create("MainWindow"))
   {
     populate_action_groups();
+    make_app_actions();
+    make_app_menu_items();
     m_newNote = utils::get_icon("note-new", 16);
     DBG_ASSERT(m_newNote, "note-new icon not found");
   }
@@ -85,15 +93,7 @@ namespace gnote {
     DBG_ASSERT(id, "merge failed");
     Gtk::Window::set_default_icon_name("gnote");
 
-
-    Gtk::ImageMenuItem *imageitem = (Gtk::ImageMenuItem*)m_ui->get_widget(
-      "/MainWindowMenubar/FileMenu/FileMenuNewNotePlaceholder/NewNote");
-    DBG_ASSERT(imageitem, "Item not found");
-    if (imageitem) {
-      imageitem->set_image(*manage(new Gtk::Image(m_newNote)));
-    }
-      
-    imageitem = (Gtk::ImageMenuItem*)m_ui->get_widget (
+    Gtk::ImageMenuItem *imageitem = (Gtk::ImageMenuItem*)m_ui->get_widget (
       "/TrayIconMenu/TrayNewNotePlaceholder/TrayNewNote");
     DBG_ASSERT(imageitem, "Item not found");
     if (imageitem) {
@@ -171,47 +171,17 @@ namespace gnote {
   void ActionManager::populate_action_groups()
   {
     Glib::RefPtr<Gtk::Action> action;
-    action = Gtk::Action::create("FileMenuAction", _("_File"));
-    m_main_window_actions->add(action);
-
-    action = Gtk::Action::create("NewNoteAction", 
-                                 Gtk::Stock::NEW, _("_New"),
-                                 _("Create a new note"));
-    m_main_window_actions->add(action, Gtk::AccelKey("<Control>N"));
-
-    action = Gtk::Action::create(
-      "OpenNoteAction", Gtk::Stock::OPEN,
-      _("_Open..."),_("Open the selected note"));
-    action->set_sensitive(false);
-    m_main_window_actions->add(action, Gtk::AccelKey( "<Control>O"));
-
-    action = Gtk::Action::create(
-      "DeleteNoteAction", Gtk::Stock::DELETE,
-      _("_Delete"),  _("Delete the selected note"));
-    action->set_sensitive(false);
-    m_main_window_actions->add(action, Gtk::AccelKey("Delete"));
-
-    action = Gtk::Action::create(
-      "CloseWindowAction", Gtk::Stock::CLOSE,
-      _("_Close"), _("Close this window"));
-    m_main_window_actions->add(action, Gtk::AccelKey("<Control>W"));
 
     action = Gtk::Action::create(
       "QuitGNoteAction", Gtk::Stock::QUIT,
       _("_Quit"), _("Quit Gnote"));
     m_main_window_actions->add(action, Gtk::AccelKey("<Control>Q"));
 
-    action = Gtk::Action::create("EditMenuAction", _("_Edit"));
-    m_main_window_actions->add(action);
-
     action = Gtk::Action::create(
       "ShowPreferencesAction", Gtk::Stock::PREFERENCES,
       _("_Preferences"), _("Gnote Preferences"));
     m_main_window_actions->add(action);
 
-    action = Gtk::Action::create("HelpMenuAction", _("_Help"));
-    m_main_window_actions->add(action);
-
     action = Gtk::Action::create("ShowHelpAction", Gtk::Stock::HELP,
       _("_Contents"), _("Gnote Help"));
     m_main_window_actions->add(action, Gtk::AccelKey("F1"));
@@ -235,11 +205,6 @@ namespace gnote {
       _("_Search All Notes"),  _("Open the Search All Notes window"));
     m_main_window_actions->add(action);
 
-    action = Gtk::Action::create(
-      "NoteSynchronizationAction",
-      _("S_ynchronize Notes"), _("Start synchronizing notes"));
-    m_main_window_actions->add(action);    
-
     m_ui->insert_action_group(m_main_window_actions);
   }
 
@@ -260,4 +225,97 @@ namespace gnote {
     return Glib::RefPtr<Gtk::Action>();      
   }
 
+  void ActionManager::make_app_actions()
+  {
+    add_app_action("new-note");
+    add_app_action("new-window");
+    add_app_action("show-preferences");
+    add_app_action("about");
+    add_app_action("help-contents");
+    add_app_action("quit");
+  }
+
+  Glib::RefPtr<Gio::SimpleAction> ActionManager::get_app_action(const std::string & name) const
+  {
+    for(std::vector<Glib::RefPtr<Gio::SimpleAction> >::const_iterator iter = m_app_actions.begin();
+        iter != m_app_actions.end(); ++iter) {
+      if((*iter)->get_name() == name) {
+        return *iter;
+      }
+    }
+    
+    return Glib::RefPtr<Gio::SimpleAction>();
+  }
+
+  void ActionManager::add_app_action(const std::string & name)
+  {
+    m_app_actions.push_back(Gio::SimpleAction::create(name));
+  }
+
+  void ActionManager::add_app_menu_item(int section, int order, const std::string & label,
+                                        const std::string & action_def)
+  {
+    m_app_menu_items.insert(std::make_pair(section, AppMenuItem(order, label, action_def)));
+  }
+
+  void ActionManager::make_app_menu_items()
+  {
+    add_app_menu_item(APP_ACTION_NEW, 100, _("_New Note"), "app.new-note");
+    add_app_menu_item(APP_ACTION_NEW, 200, _("New _Window"), "app.new-window");
+    add_app_menu_item(APP_ACTION_MANAGE, 100, _("_Preferences"), "app.show-preferences");
+    add_app_menu_item(APP_ACTION_HELP, 100, _("Help _Contents"), "app.help-contents");
+    add_app_menu_item(APP_ACTION_HELP, 200, _("_About"), "app.about");
+    add_app_menu_item(APP_ACTION_LAST, 100, _("_Quit"), "app.quit");
+  }
+
+  Glib::RefPtr<Gio::Menu> ActionManager::get_app_menu() const
+  {
+    Glib::RefPtr<Gio::Menu> menu = Gio::Menu::create();
+
+    int pos = 0;
+    Glib::RefPtr<Gio::Menu> section = make_app_menu_section(APP_ACTION_NEW);
+    if(section != 0) {
+      menu->insert_section(pos++, "", section);
+    }
+
+    section = make_app_menu_section(APP_ACTION_MANAGE);
+    if(section != 0) {
+      menu->insert_section(pos++, "", section);
+    }
+
+    section = make_app_menu_section(APP_ACTION_HELP);
+    if(section != 0) {
+      menu->insert_section(pos++, "", section);
+    }
+
+    section = make_app_menu_section(APP_ACTION_LAST);
+    if(section != 0) {
+      menu->insert_section(pos++, "", section);
+    }
+
+    return menu;
+  }
+
+  Glib::RefPtr<Gio::Menu> ActionManager::make_app_menu_section(int sec) const
+  {
+    std::pair<AppMenuItemMultiMap::const_iterator, AppMenuItemMultiMap::const_iterator>
+    range = m_app_menu_items.equal_range(sec);
+
+    Glib::RefPtr<Gio::Menu> section;
+    if(range.first != m_app_menu_items.end()) {
+      std::vector<const AppMenuItem*> menu_items;
+      for(AppMenuItemMultiMap::const_iterator iter = range.first; iter != range.second; ++iter) {
+        menu_items.push_back(&iter->second);
+      }
+      std::sort(menu_items.begin(), menu_items.end(), AppMenuItem::ptr_comparator());
+
+      section = Gio::Menu::create();
+      for(std::vector<const AppMenuItem*>::iterator iter = menu_items.begin(); iter != menu_items.end(); ++iter) {
+        section->append((*iter)->label, (*iter)->action_def);
+      }
+    }
+
+    return section;
+  }
+
 }
