@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2013 Aurimas Cernius
+ * Copyright (C) 2010-2012 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -27,6 +27,7 @@
 
 #include <iostream>
 
+#include <boost/bind.hpp>
 #include <boost/format.hpp>
 
 #include <glibmm/thread.h>
@@ -43,7 +44,6 @@
 #include "notemanager.hpp"
 #include "notewindow.hpp"
 #include "preferencesdialog.hpp"
-#include "recentchanges.hpp"
 #include "remotecontrolproxy.hpp"
 #include "utils.hpp"
 #include "xkeybinder.hpp"
@@ -106,13 +106,6 @@ namespace gnote {
   }
 
 
-  void Gnote::on_activate()
-  {
-    Gtk::Application::on_activate();
-    open_search_all();
-  }
-
-
   int Gnote::on_command_line(const Glib::RefPtr<Gio::ApplicationCommandLine> & command_line)
   {
     Gtk::Application::on_command_line(command_line);
@@ -131,7 +124,7 @@ namespace gnote {
       cmd_line.execute();
     }
     else {
-      ActionManager::obj()["ShowSearchAllNotesAction"]->activate();
+      new_main_window()->present();
     }
 
     return 0;
@@ -163,7 +156,6 @@ namespace gnote {
   {
     ActionManager & am(ActionManager::obj());
     if((m_is_background = cmd_line.background())) {
-      am["CloseWindowAction"]->set_visible(true);
       am["QuitGNoteAction"]->set_visible(false);
     }
     if(cmd_line.needs_execute()) {
@@ -195,6 +187,8 @@ namespace gnote {
       }
     }
 
+    make_app_actions();
+    make_app_menu();
     Glib::RefPtr<Gio::Settings> settings = Preferences::obj()
       .get_schema_settings(Preferences::SCHEMA_GNOTE);
     if(settings->get_boolean(Preferences::USE_STATUS_ICON)) {
@@ -203,7 +197,7 @@ namespace gnote {
     }
     else if(m_is_background) {
       // Create Search All Notes window as we need it present for application to run
-      NoteRecentChanges::get_instance(default_note_manager());
+      new_main_window();
     }
     else {
       am["ShowSearchAllNotesAction"]->activate();
@@ -214,7 +208,7 @@ namespace gnote {
   void Gnote::start_note_created(const Note::Ptr & start_note)
   {
     DBG_OUT("we will show the start note");
-    start_note->get_window()->show();
+    open_note(start_note);
   }
 
   std::string Gnote::get_note_path(const std::string & override_path)
@@ -234,30 +228,14 @@ namespace gnote {
     return note_path;
   }
 
-  bool Gnote::show_tray_icon_timeout()
-  {
-    // Setting to false and back to true is required to make it work on some tray implementations
-    m_tray_icon->set_visible(false);
-    m_tray_icon->set_visible(true);
-    return false;
-  }
-
   void Gnote::start_tray_icon()
   {
     // Create Search All Notes window as we need it present for application to run
-    NoteRecentChanges::get_instance(default_note_manager());
+    get_main_window();
 
-    if(!m_tray_icon) {
-      // Create the tray icon and run the main loop
-      m_tray_icon = Glib::RefPtr<TrayIcon>(new TrayIcon(default_note_manager()));
-      m_tray = m_tray_icon->tray();
-    }
-
-    // Tome make status icon visible on some implementations, it should be made visible
-    // after some timeout.
-    Glib::RefPtr<Glib::TimeoutSource> timeout = Glib::TimeoutSource::create(100);
-    timeout->connect(sigc::mem_fun(*this, &Gnote::show_tray_icon_timeout));
-    timeout->attach();
+    // Create the tray icon and run the main loop
+    m_tray_icon = Glib::RefPtr<TrayIcon>(new TrayIcon(default_note_manager()));
+    m_tray = m_tray_icon->tray();
   }
 
 
@@ -284,12 +262,14 @@ namespace gnote {
     bool use_status_icon = Preferences::obj()
       .get_schema_settings(Preferences::SCHEMA_GNOTE)->get_boolean(key);
     if(use_status_icon) {
-      start_tray_icon();
+      if(!m_tray_icon) {
+        m_tray_icon = Glib::RefPtr<TrayIcon>(new TrayIcon(default_note_manager()));
+      }
+      m_tray_icon->set_visible(true);
     }
     else {
-      if(m_tray_icon) {
+      if(m_tray_icon)
         m_tray_icon->set_visible(false);
-      }
       ActionManager::obj()["ShowSearchAllNotesAction"]->activate();
     }
   }
@@ -298,43 +278,21 @@ namespace gnote {
   void Gnote::setup_global_actions()
   {
     ActionManager & am(ActionManager::obj());
-    am["NewNoteAction"]->signal_activate()
-      .connect(sigc::mem_fun(*this, &Gnote::on_new_note_action));
     am["QuitGNoteAction"]->signal_activate()
-      .connect(sigc::mem_fun(*this, &Gnote::on_quit_gnote_action));
-    am["ShowPreferencesAction"]->signal_activate() 
-      .connect(sigc::mem_fun(*this, &Gnote::on_show_preferences_action));
+      .connect(sigc::mem_fun(*this, &Gnote::quit));
+    am["ShowPreferencesAction"]->signal_activate().connect(
+      boost::bind(sigc::mem_fun(*this, &Gnote::on_show_preferences_action), Glib::VariantBase()));
     am["ShowHelpAction"]->signal_activate()
-      .connect(sigc::mem_fun(*this, &Gnote::on_show_help_action));
+      .connect(boost::bind(sigc::mem_fun(*this, &Gnote::on_show_help_action), Glib::VariantBase()));
     am["ShowAboutAction"]->signal_activate()
-      .connect(sigc::mem_fun(*this, &Gnote::on_show_about_action));
+      .connect(boost::bind(sigc::mem_fun(*this, &Gnote::on_show_about_action), Glib::VariantBase()));
     am["TrayNewNoteAction"]->signal_activate()
-      .connect(sigc::mem_fun(*this, &Gnote::on_new_note_action));
+      .connect(boost::bind(sigc::mem_fun(*this, &Gnote::on_new_note_app_action), Glib::VariantBase()));
     am["ShowSearchAllNotesAction"]->signal_activate()
       .connect(sigc::mem_fun(*this, &Gnote::open_search_all));
-    am["NoteSynchronizationAction"]->signal_activate()
-      .connect(sigc::mem_fun(*this, &Gnote::open_note_sync_window));
   }
 
-  void Gnote::on_new_note_action()
-  {
-    try {
-      Note::Ptr new_note = default_note_manager().create();
-      new_note->get_window()->show();
-    }
-    catch(const std::exception & e) 
-    {
-      utils::HIGMessageDialog dialog (
-        NULL,  (GtkDialogFlags)0,
-        Gtk::MESSAGE_ERROR,
-        Gtk::BUTTONS_OK,
-        _("Cannot create new note"),
-        e.what());
-      dialog.run ();
-    }    
-  }
-
-  void Gnote::on_quit_gnote_action()
+  void Gnote::on_quit_gnote_action(const Glib::VariantBase&)
   {
     quit();
   }
@@ -346,7 +304,7 @@ namespace gnote {
   }
 
 
-  void Gnote::on_show_preferences_action()
+  void Gnote::on_show_preferences_action(const Glib::VariantBase&)
   {
     if(!m_prefsdlg) {
       m_prefsdlg = new PreferencesDialog(m_manager->get_addin_manager());
@@ -356,7 +314,7 @@ namespace gnote {
     m_prefsdlg->present();
   }
 
-  void Gnote::on_show_help_action()
+  void Gnote::on_show_help_action(const Glib::VariantBase&)
   {
     GdkScreen *cscreen = NULL;
     if(m_tray_icon) {
@@ -367,7 +325,7 @@ namespace gnote {
     utils::show_help("gnote", "", cscreen, NULL);
   }
 
-  void Gnote::on_show_about_action()
+  void Gnote::on_show_about_action(const Glib::VariantBase&)
   {
     std::vector<Glib::ustring> authors;
     authors.push_back("Aurimas Černius <aurisc4@gmail.com>");
@@ -389,7 +347,7 @@ namespace gnote {
     about.set_program_name("Gnote");
     about.set_version(VERSION);
     about.set_logo(utils::get_icon("gnote", 48));
-    about.set_copyright(_("Copyright \xc2\xa9 2010-2013 Aurimas Cernius\n"
+    about.set_copyright(_("Copyright \xc2\xa9 2010-2012 Aurimas Cernius\n"
                           "Copyright \xc2\xa9 2009-2011 Debarshi Ray\n"
                           "Copyright \xc2\xa9 2009 Hubert Figuiere\n"
                           "Copyright \xc2\xa9 2004-2009 the Tomboy original authors."));
@@ -409,7 +367,7 @@ namespace gnote {
     about.set_documenters(documenters);
     about.set_translator_credits(translators);
 //      about.set_icon_name("gnote");
-    NoteRecentChanges *recent_changes = NoteRecentChanges::get_instance();
+    NoteRecentChanges::Ptr recent_changes = get_main_window();
     if(recent_changes && recent_changes->get_visible()) {
       about.set_transient_for(*recent_changes);
       recent_changes->present();
@@ -417,12 +375,64 @@ namespace gnote {
     about.run();
   }
 
-  void Gnote::open_search_all()
+  NoteRecentChanges::Ptr Gnote::new_main_window()
   {
-    NoteRecentChanges::get_instance(default_note_manager())->present();
+    NoteRecentChanges::Ptr win = NoteRecentChanges::create(default_note_manager());
+    std::list<NoteRecentChanges::Ptr>::iterator pos = m_main_windows.insert(m_main_windows.end(), win);
+    win->signal_hide().connect(boost::bind(sigc::mem_fun(*this, &Gnote::on_main_window_closed), pos));
+    add_window(*win);
+    return win;
   }
 
-  void Gnote::open_note_sync_window()
+  NoteRecentChanges::Ptr Gnote::get_main_window()
+  {
+    for(std::list<NoteRecentChanges::Ptr>::iterator iter = m_main_windows.begin();
+        iter != m_main_windows.end(); ++iter) {
+      return *iter;
+    }
+
+    NoteRecentChanges::Ptr win = new_main_window();
+    return win;
+  }
+
+  void Gnote::on_main_window_closed(std::list<NoteRecentChanges::Ptr>::iterator pos)
+  {
+    remove_window(**pos);
+    m_main_windows.erase(pos);
+
+    // if background mode, we need to have a window, to prevent quit
+    if(m_is_background && !m_main_windows.size()) {
+      new_main_window();
+    }
+  }
+
+  NoteRecentChanges::Ptr Gnote::get_window_for_note()
+  {
+    NoteRecentChanges::Ptr window;
+    //Find first visible window or any window otherwise
+    for(std::list<NoteRecentChanges::Ptr>::iterator iter = m_main_windows.begin();
+        iter != m_main_windows.end(); ++iter) {
+      if((*iter)->get_visible()) {
+        window = *iter;
+        break;
+      }
+      if(!window) {
+        window = *iter;
+      }
+    }
+    if(!window) {
+      window = new_main_window();
+    }
+
+    return window;
+  }
+
+  void Gnote::open_search_all()
+  {
+    get_main_window()->present();
+  }
+
+  void Gnote::open_note_sync_window(const Glib::VariantBase&)
   {
     if(m_sync_dlg == 0) {
       m_sync_dlg = sync::SyncDialog::create();
@@ -440,6 +450,76 @@ namespace gnote {
   }
 
 
+  void Gnote::make_app_actions()
+  {
+    ActionManager & am(ActionManager::obj());
+    am.get_app_action("new-note")->signal_activate().connect(sigc::mem_fun(*this, &Gnote::on_new_note_app_action));
+    am.get_app_action("new-window")->signal_activate().connect(sigc::mem_fun(*this, &Gnote::on_new_window_action));
+    am.get_app_action("show-preferences")->signal_activate().connect(
+      sigc::mem_fun(*this, &Gnote::on_show_preferences_action));
+    am.get_app_action("sync-notes")->signal_activate().connect(sigc::mem_fun(*this, &Gnote::open_note_sync_window));
+    am.get_app_action("help-contents")->signal_activate().connect(sigc::mem_fun(*this, &Gnote::on_show_help_action));
+    am.get_app_action("about")->signal_activate().connect(sigc::mem_fun(*this, &Gnote::on_show_about_action));
+    am.get_app_action("quit")->signal_activate().connect(sigc::mem_fun(*this, &Gnote::on_quit_gnote_action));
+
+    add_app_actions(am.get_app_actions());
+  }
+
+
+  void Gnote::add_app_actions(const std::vector<Glib::RefPtr<Gio::SimpleAction> > & actions)
+  {
+    for(std::vector<Glib::RefPtr<Gio::SimpleAction> >::const_iterator iter = actions.begin();
+        iter != actions.end(); ++iter) {
+      add_action(*iter);
+    }
+  }
+
+
+  void Gnote::on_new_window_action(const Glib::VariantBase&)
+  {
+    new_main_window()->present();
+  }
+
+
+  NoteRecentChanges *Gnote::get_active_window()
+  {
+    std::vector<Gtk::Window*> windows = Gtk::Window::list_toplevels();
+    for(std::vector<Gtk::Window*>::iterator iter = windows.begin(); iter != windows.end(); ++iter) {
+      if((*iter)->property_is_active()) {
+        return dynamic_cast<NoteRecentChanges*>(*iter);
+      }
+    }
+
+    return NULL;
+  }
+
+
+  void Gnote::on_new_note_app_action(const Glib::VariantBase&)
+  {
+    NoteRecentChanges *rc = get_active_window();
+    if(rc) {
+      rc->new_note();
+    }
+    else {
+      get_main_window()->new_note();
+    }
+  }
+
+
+  void Gnote::open_note(const Note::Ptr & note)
+  {
+    NoteRecentChanges::Ptr main_window = get_window_for_note();
+    main_window->present_note(note);
+    main_window->present();
+  }
+
+
+  void Gnote::make_app_menu()
+  {
+    set_app_menu(ActionManager::obj().get_app_menu());
+  }
+
+
   std::string Gnote::cache_dir()
   {
     return Glib::get_user_cache_dir() + "/gnote";
@@ -577,7 +657,7 @@ namespace gnote {
       else {
         execute(remote);
       }
-      Gnote::obj().on_quit_gnote_action();
+      Gnote::obj().quit();
     }
     return 0;
   }
