@@ -34,6 +34,7 @@
 #include "base/singleton.hpp"
 #include "actionmanager.hpp"
 #include "keybinder.hpp"
+#include "recentchanges.hpp"
 #include "remotecontrolproxy.hpp"
 #include "tray.hpp"
 #include "synchronization/syncdialog.hpp"
@@ -125,14 +126,16 @@ public:
   void setup_global_actions();
   void start_tray_icon();
 
-  void on_new_note_action();
-  void on_quit_gnote_action();
+  void on_quit_gnote_action(const Glib::VariantBase&);
   void on_preferences_response(int res);
-  void on_show_preferences_action();
-  void on_show_help_action();
-  void on_show_about_action();
+  void on_show_preferences_action(const Glib::VariantBase&);
+  void on_show_help_action(const Glib::VariantBase&);
+  void on_show_about_action(const Glib::VariantBase&);
+  NoteRecentChanges::Ptr new_main_window();
+  NoteRecentChanges::Ptr get_main_window();
+  NoteRecentChanges::Ptr get_window_for_note();
   void open_search_all();
-  void open_note_sync_window();
+  void open_note_sync_window(const Glib::VariantBase&);
 
   static std::string cache_dir();
   static std::string conf_dir();
@@ -162,8 +165,8 @@ public:
     {
       return m_sync_dlg;
     }
+  void open_note(const Note::Ptr & note);
 protected:
-  virtual void on_activate();
   virtual int on_command_line(const Glib::RefPtr<Gio::ApplicationCommandLine> & command_line);
   virtual void on_startup();
   virtual void on_window_removed(Gtk::Window *window);
@@ -177,7 +180,13 @@ private:
   void common_init();
   void end_main(bool bus_aquired, bool name_acquired);
   void on_sync_dialog_response(int response_id);
-  bool show_tray_icon_timeout();
+  void on_main_window_closed(std::list<NoteRecentChanges::Ptr>::iterator pos);
+  void make_app_actions();
+  void add_app_actions(const std::vector<Glib::RefPtr<Gio::SimpleAction> > & actions);
+  void make_app_menu();
+  void on_new_window_action(const Glib::VariantBase&);
+  void on_new_note_app_action(const Glib::VariantBase&);
+  NoteRecentChanges *get_active_window();
 
   NoteManager *m_manager;
   IKeybinder  *m_keybinder;
@@ -188,6 +197,7 @@ private:
   PreferencesDialog *m_prefsdlg;
   GnoteCommandLine cmd_line;
   sync::SyncDialog::Ptr m_sync_dlg;
+  std::list<NoteRecentChanges::Ptr> m_main_windows;
 };
 
 
