@@ -278,6 +278,7 @@ namespace gnote {
     , m_save_needed(false)
     , m_is_deleting(false)
     , m_enabled(true)
+    , m_note_window_embeded(false)
     , m_focus_widget(NULL)
     , m_manager(_manager)
     , m_window(NULL)
@@ -377,7 +378,9 @@ namespace gnote {
     }
 
     if (m_window) {
-      m_window->hide ();
+      if(m_window->host()) {
+        m_window->host()->unembed_widget(*m_window);
+      }
       delete m_window; 
       m_window = NULL;
     }
@@ -414,7 +417,7 @@ namespace gnote {
     catch (const sharp::Exception & e) {
       // Probably IOException or UnauthorizedAccessException?
       ERR_OUT("Exception while saving note: %s", e.what());
-      show_io_error_dialog(m_window);
+      show_io_error_dialog(dynamic_cast<Gtk::Window*>(m_window->host()));
     }
 
     m_signal_saved(shared_from_this());
@@ -464,43 +467,6 @@ namespace gnote {
     queue_save(NO_CHANGE);
   }
 
-  void Note::on_buffer_mark_deleted(const Glib::RefPtr<Gtk::TextBuffer::Mark> &)
-  {
-    Gtk::TextIter start, end;
-    if(m_data.data().selection_bound_position() != m_data.data().cursor_position()
-       && !m_buffer->get_selection_bounds(start, end)) {
-      DBG_OUT("selection removed");
-      m_data.data().set_cursor_position(m_buffer->get_insert()->get_iter().get_offset());
-      m_data.data().set_selection_bound_position(NoteData::s_noPosition);
-      queue_save(NO_CHANGE);
-    }
-  }
-
-
-  bool Note::on_window_configure(GdkEventConfigure * /*ev*/)
-  {
-    int cur_x, cur_y, cur_width, cur_height;
-
-    // Ignore events when maximized.  We don't want notes
-    // popping up maximized the next run.
-    if ((m_window->get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED) != 0)
-      return false;
-
-    m_window->get_position(cur_x, cur_y);
-    m_window->get_size(cur_width, cur_height);
-
-    if (m_data.data().x() == cur_x &&
-        m_data.data().y() == cur_y &&
-        m_data.data().width() == cur_width &&
-        m_data.data().height() == cur_height)
-      return false;
-
-    m_data.data().set_position_extent(cur_x, cur_y, cur_width, cur_height);
-
-    DBG_OUT("WindowConfigureEvent queueing save");
-    queue_save(NO_CHANGE);
-    return false;
-  }
 
   bool Note::on_window_destroyed(GdkEventAny * /*ev*/)
   {
@@ -666,7 +632,7 @@ namespace gnote {
   {
     if (m_data.data().title() != new_title) {
       if (m_window) {
-        m_window->set_title(new_title);
+        m_window->set_name(new_title);
       }
 
       std::string old_title = m_data.data().title();
@@ -814,7 +780,7 @@ namespace gnote {
   {
     if (m_data.data().title() != newTitle) {
       if (m_window) {
-        m_window->set_title(newTitle);
+        m_window->set_name(newTitle);
       }
 
       m_data.data().title() = newTitle;
@@ -1027,8 +993,6 @@ namespace gnote {
         sigc::mem_fun(*this, &Note::on_buffer_tag_removed));
       m_buffer->signal_mark_set().connect(
         sigc::mem_fun(*this, &Note::on_buffer_mark_set));
-      m_buffer->signal_mark_deleted().connect(
-        sigc::mem_fun(*this, &Note::on_buffer_mark_deleted));
     }
     return m_buffer;
   }
@@ -1040,21 +1004,23 @@ namespace gnote {
       m_window = new NoteWindow(*this);
       m_window->signal_delete_event().connect(
         sigc::mem_fun(*this, &Note::on_window_destroyed));
-      m_window->signal_configure_event().connect(
-        sigc::mem_fun(*this, &Note::on_window_configure), false);
-      m_window->signal_show().connect(
-        sigc::mem_fun(*this, &Note::on_window_show));
-      m_window->signal_hide().connect(
-        sigc::mem_fun(*this, &Note::on_window_hide));
 
       m_window->editor()->set_sensitive(enabled());
-      if (m_data.data().has_extent())
-        m_window->set_default_size(m_data.data().width(),
-                                   m_data.data().height());
+      if(m_data.data().has_extent()) {
+        m_window->set_size(m_data.data().width(), m_data.data().height());
+      }
+      if(m_data.data().has_position()) {
+        m_window->set_position(m_data.data().x(), m_data.data().y());
+      }
 
-      if (m_data.data().has_position())
-        m_window->move(m_data.data().x(), m_data.data().y());
+      m_window->signal_embeded.connect(sigc::mem_fun(*this, &Note::on_note_window_embeded));
+    }
+    return m_window;
+  }
 
+  void Note::on_note_window_embeded()
+  {
+    if(!m_note_window_embeded) {
       // This is here because emiting inside
       // OnRealized causes segfaults.
       m_signal_opened(*this);
@@ -1062,22 +1028,10 @@ namespace gnote {
       // Add any child widgets if any exist now that
       // the window is showing.
       process_child_widget_queue();
+      m_note_window_embeded = true;
     }
-    return m_window;
   }
 
-  void Note::on_window_show()
-  {
-    Gnote::obj().add_window(*m_window);
-  }
-
-
-  void Note::on_window_hide()
-  {
-    Gnote::obj().remove_window(*m_window);
-  }
-
-
   bool Note::is_special() const
   { 
     return (m_manager.start_note_uri() == m_data.data().uri());
@@ -1148,12 +1102,15 @@ namespace gnote {
   {
     m_enabled = is_enabled;
     if(m_window) {
-      if(!m_enabled) {
-        m_focus_widget = m_window->get_focus();
-      }
-      m_window->set_sensitive(m_enabled);
-      if(m_enabled) {
-        m_window->set_focus(*m_focus_widget);
+      Gtk::Window *window = dynamic_cast<Gtk::Window*>(m_window->host());
+      if(window) {
+        if(!m_enabled) {
+          m_focus_widget = window->get_focus();
+        }
+        window->set_sensitive(m_enabled);
+        if(m_enabled) {
+          window->set_focus(*m_focus_widget);
+        }
       }
     }
   }
