@@ -376,8 +376,6 @@ private:
                              const Gtk::TextBuffer::iterator &);
   void on_buffer_mark_set(const Gtk::TextBuffer::iterator & iter,
                           const Glib::RefPtr<Gtk::TextBuffer::Mark> & insert);
-  bool on_window_configure(GdkEventConfigure *ev);
-  void on_buffer_mark_deleted(const Glib::RefPtr<Gtk::TextBuffer::Mark> & mark);
   bool on_window_destroyed(GdkEventAny *ev);
   void on_save_timeout();
   void process_child_widget_queue();
@@ -386,12 +384,10 @@ private:
                     const Ptr & renamed);
   void remove_links(const std::string & old_title,
                     const Ptr & renamed);
+  void on_note_window_embeded();
 
   Note(NoteData * data, const std::string & filepath, NoteManager & manager);
 
-  void on_window_show();
-  void on_window_hide();
-
   struct ChildWidgetData
   {
     ChildWidgetData(const Glib::RefPtr<Gtk::TextChildAnchor> & _anchor,
@@ -409,6 +405,7 @@ private:
   bool                       m_save_needed;
   bool                       m_is_deleting;
   bool                       m_enabled;
+  bool                       m_note_window_embeded;
 
   Gtk::Widget               *m_focus_widget;
   NoteManager               &m_manager;
