@@ -52,8 +52,13 @@
 namespace gnote {
 
   NoteWindow::NoteWindow(Note & note)
-    : ForcedPresentWindow(note.get_title())
+    : Gtk::VBox(false, 2)
     , m_note(note)
+    , m_name(note.get_title())
+    , m_height(360)
+    , m_width(450)
+    , m_x(-1)
+    , m_y(-1)
     , m_global_keys(NULL)
   {
     m_template_tag = TagManager::obj().get_or_create_system_tag(TagManager::TEMPLATE_NOTE_SYSTEM_TAG);
@@ -61,16 +66,7 @@ namespace gnote {
     m_template_save_selection_tag = TagManager::obj().get_or_create_system_tag(TagManager::TEMPLATE_NOTE_SAVE_SELECTION_SYSTEM_TAG);
     m_template_save_title_tag = TagManager::obj().get_or_create_system_tag(TagManager::TEMPLATE_NOTE_SAVE_TITLE_SYSTEM_TAG);
 
-//    get_window()->set_icon_name("gnote");
-    set_default_size(450, 360);
-    set_resizable(true);
-
-    m_accel_group = Gtk::AccelGroup::create();
-    add_accel_group(m_accel_group);
-
-    m_text_menu 
-      = Gtk::manage(new NoteTextMenu(m_accel_group, 
-                                     note.get_buffer(), note.get_buffer()->undoer()));
+    m_text_menu = Gtk::manage(new NoteTextMenu(note.get_buffer(), note.get_buffer()->undoer()));
 
     // Add the Find menu item to the toolbar Text menu.  It
     // should only show up in the toplevel Text menu, since
@@ -80,17 +76,11 @@ namespace gnote {
     spacer->show ();
     m_text_menu->append(*spacer);
 
-    Gtk::ImageMenuItem *find_item =
-      manage(new Gtk::ImageMenuItem (_("Find in This Note")));
-    find_item->set_image(*manage(new Gtk::Image (Gtk::Stock::FIND, Gtk::ICON_SIZE_MENU)));
-    find_item->signal_activate().connect(sigc::mem_fun(*this, &NoteWindow::find_button_clicked));
-    find_item->add_accelerator ("activate",
-                                m_accel_group,
-                                GDK_KEY_F,
-                                Gdk::CONTROL_MASK,
-                                Gtk::ACCEL_VISIBLE);
-    find_item->show ();
-    m_text_menu->append(*find_item);
+    m_find_item = manage(new Gtk::ImageMenuItem(_("Find in This Note")));
+    m_find_item->set_image(*manage(new Gtk::Image (Gtk::Stock::FIND, Gtk::ICON_SIZE_MENU)));
+    m_find_item->signal_activate().connect(sigc::mem_fun(*this, &NoteWindow::find_button_clicked));
+    m_find_item->show ();
+    m_text_menu->append(*m_find_item);
 
     m_plugin_menu = manage(make_plugin_menu());
 
@@ -127,81 +117,12 @@ namespace gnote {
     m_find_bar->set_no_show_all(true);
     m_find_bar->signal_hide().connect(sigc::mem_fun(*this, &NoteWindow::find_bar_hidden));
 
-    Gtk::VBox *box = manage(new Gtk::VBox (false, 2));
-    box->pack_start(*m_toolbar, false, false, 0);
-    box->pack_start(*m_template_widget, false, false, 0);
-    box->pack_start(*m_editor_window, true, true, 0);
-    box->pack_start(*m_find_bar, false, false, 0);
-
-    box->show();
-
-    // Don't set up Ctrl-W or Ctrl-N if Emacs is in use
-    bool using_emacs = false;
-    Glib::RefPtr<Gio::Settings> desktop_settings = Preferences::obj()
-      .get_schema_settings(Preferences::SCHEMA_DESKTOP_GNOME_INTERFACE);
-    if(desktop_settings) {
-      std::string gtk_key_theme = 
-        desktop_settings->get_string(Preferences::DESKTOP_GNOME_KEY_THEME);
-    if (!gtk_key_theme.empty() && (gtk_key_theme == "Emacs"))
-      using_emacs = true;
-    }
-
-    // NOTE: Since some of our keybindings are only
-    // available in the context menu, and the context menu
-    // is created on demand, register them with the
-    // global keybinder
-    m_global_keys = new utils::GlobalKeybinder (m_accel_group);
-
-    // Close window (Ctrl-W)
-    if (!using_emacs)
-      m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::close_window_handler),
-                                      GDK_KEY_W,
-                                      Gdk::CONTROL_MASK,
-                                      Gtk::ACCEL_VISIBLE);
-
-    // Escape has been moved to be handled by a KeyPress Handler so that
-    // Escape can be used to close the FindBar.
-
-    // Close all windows on current Desktop (Ctrl-Q)
-    m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::close_all_windows_handler),
-                                   GDK_KEY_Q,
-                                   Gdk::CONTROL_MASK,
-                                   Gtk::ACCEL_VISIBLE);
-
-    // Find Next (Ctrl-G)
-    m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::find_next_activate),
-                                 GDK_KEY_G,
-                                 Gdk::CONTROL_MASK,
-                                 Gtk::ACCEL_VISIBLE);
-
-    // Find Previous (Ctrl-Shift-G)
-    m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::find_previous_activate),
-                                 GDK_KEY_G, (Gdk::CONTROL_MASK | Gdk::SHIFT_MASK),
-                                 Gtk::ACCEL_VISIBLE);
-
-    // Open Help (F1)
-    m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::open_help_activate),
-                                    GDK_KEY_F1, (Gdk::ModifierType)0, (Gtk::AccelFlags)0);
-
-    // Create a new note
-    if (!using_emacs)
-      m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::create_new_note),
-                                     GDK_KEY_N, Gdk::CONTROL_MASK,
-                                     Gtk::ACCEL_VISIBLE);
+    pack_start(*m_toolbar, false, false, 0);
+    pack_start(*m_template_widget, false, false, 0);
+    pack_start(*m_editor_window, true, true, 0);
+    pack_start(*m_find_bar, false, false, 0);
 
     signal_key_press_event().connect(sigc::mem_fun(*this, &NoteWindow::on_key_pressed));
-
-
-    // Increase Indent
-    m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::change_depth_right_handler),
-                                   GDK_KEY_Right, Gdk::MOD1_MASK,
-                                   Gtk::ACCEL_VISIBLE);
-
-    // Decrease Indent
-    m_global_keys->add_accelerator (sigc::mem_fun(*this, &NoteWindow::change_depth_left_handler),
-                                   GDK_KEY_Left, Gdk::MOD1_MASK,
-                                   Gtk::ACCEL_VISIBLE);
-    add(*box);
   }
 
 
@@ -216,22 +137,126 @@ namespace gnote {
   }
 
 
-  bool NoteWindow::on_delete_event(GdkEventAny * /*ev*/)
+  std::string NoteWindow::get_name() const
   {
-    close_window_handler();
-    return true;
+    return m_name;
   }
 
-  void NoteWindow::on_hide()
+  void NoteWindow::embed(utils::EmbedableWidgetHost *h)
   {
-    utils::ForcedPresentWindow::on_hide();
+    //remove from previous host, if any
+    if(host()) {
+      host()->unembed_widget(*this);
+    }
+    utils::EmbedableWidget::embed(h);
+  }
 
-    // Workaround Gtk bug, where adding or changing Widgets
-    // while the Window is hidden causes it to be reshown at
-    // 0,0...
-    int x, y;
-    get_position(x, y);
-    move(x, y);
+  void NoteWindow::foreground()
+  {
+    //addins may add accelarators, so accel group must be there
+    utils::EmbedableWidgetHost *current_host = host();
+    Gtk::Window *parent = dynamic_cast<Gtk::Window*>(current_host);
+    if(parent) {
+      add_accel_group(*parent);
+    }
+
+    utils::EmbedableWidget::foreground();
+    if(parent) {
+      parent->set_default_size(m_width, m_height);
+      if((parent->get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED) == 0 && parent->get_visible()) {
+        parent->get_window()->resize(m_width, m_height);
+      }
+      if(m_x >= 0 && m_y >= 0 && !current_host->running()) {
+        parent->move(m_x, m_y);
+      }
+      parent->set_focus(*m_editor);
+    }
+    m_editor->scroll_to(m_editor->get_buffer()->get_insert());
+  }
+
+  void NoteWindow::background()
+  {
+    utils::EmbedableWidget::background();
+    Gtk::Window *parent = dynamic_cast<Gtk::Window*>(host());
+    if(!parent) {
+      return;
+    }
+    remove_accel_group(*parent);
+    if((parent->get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED) == 0) {
+      int cur_x, cur_y, cur_width, cur_height;
+      parent->get_position(cur_x, cur_y);
+      parent->get_size(cur_width, cur_height);
+
+      if(!(m_note.data().x() == cur_x && m_note.data().y() == cur_y
+           && m_note.data().width() == cur_width
+           && m_note.data().height() == cur_height)) {
+        m_note.data().set_position_extent(cur_x, cur_y, cur_width, cur_height);
+        m_width = cur_width;
+        m_height = cur_height;
+
+        DBG_OUT("WindowConfigureEvent queueing save");
+        m_note.queue_save(NO_CHANGE);
+      }
+    }
+  }
+
+  void NoteWindow::add_accel_group(Gtk::Window & window)
+  {
+    if(!m_accel_group) {
+      m_accel_group = Gtk::AccelGroup::create();
+      window.add_accel_group(m_accel_group);
+      m_text_menu->set_accel_group(m_accel_group);
+      m_find_item->add_accelerator("activate", m_accel_group,
+                                   GDK_KEY_F,
+                                   Gdk::CONTROL_MASK,
+                                   Gtk::ACCEL_VISIBLE);
+      m_link_button->add_accelerator("clicked", m_accel_group,
+                                     GDK_KEY_L, Gdk::CONTROL_MASK,
+                                     Gtk::ACCEL_VISIBLE);
+
+      if(!m_global_keys) {
+        // NOTE: Since some of our keybindings are only
+        // available in the context menu, and the context menu
+        // is created on demand, register them with the
+        // global keybinder
+        m_global_keys = new utils::GlobalKeybinder(m_accel_group);
+
+        // Find Next (Ctrl-G)
+        m_global_keys->add_accelerator(sigc::mem_fun(*this, &NoteWindow::find_next_activate),
+                                       GDK_KEY_G,
+                                       Gdk::CONTROL_MASK,
+                                       Gtk::ACCEL_VISIBLE);
+
+        // Find Previous (Ctrl-Shift-G)
+        m_global_keys->add_accelerator(sigc::mem_fun(*this, &NoteWindow::find_previous_activate),
+                                       GDK_KEY_G, (Gdk::CONTROL_MASK | Gdk::SHIFT_MASK),
+                                       Gtk::ACCEL_VISIBLE);
+
+        // Open Help (F1)
+        m_global_keys->add_accelerator(sigc::mem_fun(*this, &NoteWindow::open_help_activate),
+                                       GDK_KEY_F1, (Gdk::ModifierType)0, (Gtk::AccelFlags)0);
+
+        // Increase Indent
+        m_global_keys->add_accelerator(sigc::mem_fun(*this, &NoteWindow::change_depth_right_handler),
+                                       GDK_KEY_Right, Gdk::MOD1_MASK,
+                                       Gtk::ACCEL_VISIBLE);
+
+        // Decrease Indent
+        m_global_keys->add_accelerator(sigc::mem_fun(*this, &NoteWindow::change_depth_left_handler),
+                                      GDK_KEY_Left, Gdk::MOD1_MASK,
+                                      Gtk::ACCEL_VISIBLE);
+      }
+    }
+    else {
+      window.add_accel_group(m_accel_group);
+    }
+  }
+
+  void NoteWindow::remove_accel_group(Gtk::Window & window)
+  {
+    if(m_accel_group) {
+      window.remove_accel_group(m_accel_group);
+    }
   }
 
 
@@ -241,49 +266,10 @@ namespace gnote {
       if (m_find_bar && m_find_bar->get_visible()) {
         m_find_bar->hide();
       }
-      else if (Preferences::obj().get_schema_settings(Preferences::SCHEMA_GNOTE)->get_boolean(
-                 Preferences::ENABLE_CLOSE_NOTE_ON_ESCAPE)) {
-        close_window_handler();
-      }
-      return true;
     }
     return false;
   }
 
-  // FIXME: Need to just emit a delete event, and do this work in
-  // the default delete handler, so that plugins can attach to
-  // delete event and have it always work.
-  void NoteWindow::close_window_handler()
-  {
-    // Unmaximize before hiding to avoid reopening
-    // pseudo-maximized
-    if ((get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED) != 0) {
-      unmaximize();
-    }
-    hide();
-  }
-
-
-  void NoteWindow::close_all_windows_handler()
-  {
-    int workspace = tomboy_window_get_workspace(gobj());
-    
-    for(Note::List::const_iterator iter = m_note.manager().get_notes().begin();
-        iter != m_note.manager().get_notes().end(); ++iter) {
-      const Note::Ptr & note(*iter);
-      if (!note->is_opened())
-        continue;
-
-      // Close windows on the same workspace, or all
-      // open windows if no workspace.
-      if ((workspace < 0) ||
-          (tomboy_window_get_workspace (note->get_window()->gobj()) == workspace)) {
-        note->get_window()->close_window_handler();
-      }
-    }
-  }
-  
-
     // Delete this Note.
     //
 
@@ -292,7 +278,7 @@ namespace gnote {
     // Prompt for note deletion
     std::list<Note::Ptr> single_note_list;
     single_note_list.push_back(m_note.shared_from_this());
-    noteutils::show_deletion_dialog(single_note_list, this);
+    noteutils::show_deletion_dialog(single_note_list, dynamic_cast<Gtk::Window*>(host()));
   }
 
   void NoteWindow::on_selection_mark_set(const Gtk::TextIter&, const Glib::RefPtr<Gtk::TextMark>&)
@@ -325,15 +311,6 @@ namespace gnote {
     Gtk::MenuItem *spacer1 = manage(new Gtk::SeparatorMenuItem());
     spacer1->show ();
 
-    Gtk::ImageMenuItem *search = manage(new Gtk::ImageMenuItem(
-                                          _("_Search All Notes"), true));
-    search->set_image(*manage(new Gtk::Image (Gtk::Stock::FIND, Gtk::ICON_SIZE_MENU)));
-    search->signal_activate().connect(sigc::mem_fun(*this, &NoteWindow::search_button_clicked));
-    search->add_accelerator ("activate", m_accel_group, GDK_KEY_F,
-                             (Gdk::CONTROL_MASK |  Gdk::SHIFT_MASK),
-                             Gtk::ACCEL_VISIBLE);
-    search->show();
-
     Gtk::ImageMenuItem *link = manage(new Gtk::ImageMenuItem(_("_Link to New Note"), true));
     link->set_image(*manage(new Gtk::Image (Gtk::Stock::JUMP_TO, Gtk::ICON_SIZE_MENU)));
     link->set_sensitive(!m_note.get_buffer()->get_selection().empty());
@@ -346,8 +323,7 @@ namespace gnote {
     text_item->set_image(*manage(new Gtk::Image(
                                    Gtk::Stock::SELECT_FONT, 
                                    Gtk::ICON_SIZE_MENU)));
-    text_item->set_submenu(*manage(new NoteTextMenu(m_accel_group,
-                                                    m_note.get_buffer(),
+    text_item->set_submenu(*manage(new NoteTextMenu(m_note.get_buffer(),
                                                     m_note.get_buffer()->undoer())));
     text_item->show();
 
@@ -363,28 +339,6 @@ namespace gnote {
     menu->prepend(*text_item);
     menu->prepend(*find_item);
     menu->prepend(*link);
-    menu->prepend(*search);
-
-    Gtk::MenuItem *close_all =
-      manage(new Gtk::MenuItem(_("Clos_e All Notes"), true));
-    close_all->signal_activate().connect(
-      sigc::mem_fun(*this, &NoteWindow::close_all_windows_handler));
-    close_all->add_accelerator("activate", m_accel_group,
-                               GDK_KEY_Q, Gdk::CONTROL_MASK, Gtk::ACCEL_VISIBLE);
-    close_all->show();
-
-    Gtk::ImageMenuItem *close_window = manage(
-      new Gtk::ImageMenuItem(_("_Close"), true));
-    close_window->set_image(*manage(new Gtk::Image(Gtk::Stock::CLOSE, Gtk::ICON_SIZE_MENU)));
-    close_window->signal_activate().connect(
-      sigc::mem_fun(*this, &NoteWindow::close_window_handler));
-    close_window->add_accelerator("activate", m_accel_group,
-                                  GDK_KEY_W,
-                                  Gdk::CONTROL_MASK, Gtk::ACCEL_VISIBLE);
-    close_window->show();
-
-    menu->append(*close_all);
-    menu->append(*close_window);
   }
   
   //
@@ -397,22 +351,6 @@ namespace gnote {
   {
     Gtk::Toolbar *tb = new Gtk::Toolbar();
 
-    Gtk::ToolButton *search = manage(new Gtk::ToolButton (
-                                       *manage(new Gtk::Image(
-                                                 Gtk::Stock::FIND, 
-                                                 tb->get_icon_size())
-                                         ), _("Search")));
-    search->set_use_underline(true);
-    search->set_is_important(true);
-    search->signal_clicked().connect(sigc::mem_fun(*this, &NoteWindow::search_button_clicked));
-    search->set_tooltip_text(_("Search your notes (Ctrl-Shift-F)"));
-    search->add_accelerator("clicked", m_accel_group,
-                             GDK_KEY_F,
-                             (Gdk::CONTROL_MASK | Gdk::SHIFT_MASK),
-                             Gtk::ACCEL_VISIBLE);
-    search->show_all();
-    tb->insert(*search, -1);
-
     m_link_button = manage(new Gtk::ToolButton(
                              *manage(new Gtk::Image (Gtk::Stock::JUMP_TO, tb->get_icon_size())),
                              _("Link")));
@@ -422,9 +360,6 @@ namespace gnote {
     m_link_button->signal_clicked().connect(
       sigc::mem_fun(*this, &NoteWindow::link_button_clicked));
     m_link_button->set_tooltip_text(_("Link selected text to a new note (Ctrl-L)"));
-    m_link_button->add_accelerator("clicked", m_accel_group,
-                                   GDK_KEY_L, Gdk::CONTROL_MASK,
-                                   Gtk::ACCEL_VISIBLE);
     m_link_button->show_all();
     tb->insert(*m_link_button, -1);
 
@@ -463,27 +398,11 @@ namespace gnote {
     }
     tb->insert(*manage(new Gtk::SeparatorToolItem()), -1);
 
-#if 0    // Disable because of bug 585610
-    Gtk::ImageMenuItem *item =
-      manage(new Gtk::ImageMenuItem (_("Synchronize Notes")));
-    item->set_image(*manage(new Gtk::Image(Gtk::Stock::CONVERT, Gtk::ICON_SIZE_MENU)));
-    item->signal_activate().connect(
-      sigc::mem_fun(*this, &NoteWindow::sync_item_selected));
-    item->show();
-    m_plugin_menu->add(*item);
-#endif
-
     tb->show_all();
     return tb;
   }
 
 
-  void NoteWindow::sync_item_selected ()
-  {
-    ActionManager::obj()["NoteSynchronizationAction"]->activate();
-  }
-
-  
   //
   // This menu can be
   // populated by individual plugins using
@@ -688,7 +607,7 @@ namespace gnote {
         match = m_note.manager().create(select);
       } 
       catch (const sharp::Exception & e) {
-        utils::HIGMessageDialog dialog(this,
+        utils::HIGMessageDialog dialog(dynamic_cast<Gtk::Window*>(host()),
           GTK_DIALOG_DESTROY_WITH_PARENT,
           Gtk::MESSAGE_ERROR,  Gtk::BUTTONS_OK,
           _("Cannot create note"), e.what());
@@ -703,26 +622,13 @@ namespace gnote {
       m_note.get_buffer()->apply_tag(m_note.get_tag_table()->get_link_tag(), start, end);
     }
 
-    match->get_window()->present();
+    host()->embed_widget(*match->get_window());
   }
 
   void NoteWindow::open_help_activate()
   {
-    utils::show_help("gnote", "editing-notes", get_screen()->gobj(), this);
-  }
-
-  void NoteWindow::create_new_note ()
-  {
-    ActionManager::obj()["NewNoteAction"]->activate();
-  }
-
-  void NoteWindow::search_button_clicked()
-  {
-    NoteRecentChanges *search = NoteRecentChanges::get_instance(m_note.manager());
-    if (!m_note.get_buffer()->get_selection().empty()) {
-      search->set_search_text(m_note.get_buffer()->get_selection());
-    }
-    search->present();
+    utils::show_help("gnote", "editing-notes", get_screen()->gobj(),
+                     dynamic_cast<Gtk::Window*>(host()));
   }
 
   void NoteWindow::change_depth_right_handler()
@@ -735,14 +641,7 @@ namespace gnote {
     Glib::RefPtr<NoteBuffer>::cast_static(m_editor->get_buffer())->change_cursor_depth_directional(false);
   }
 
-  void NoteWindow::on_show()
-  {
-    utils::ForcedPresentWindow::on_show();
-    // Make sure the cursor position is visible
-    m_editor->scroll_to (m_editor->get_buffer()->get_insert());
-  }
 
-  
   NoteFindBar::NoteFindBar(Note & note)
     : Gtk::HBox(false, 0)
     , m_note(note)
@@ -1197,8 +1096,7 @@ namespace gnote {
   // Menu for font style and size, and set the active radio
   // menuitem depending on the cursor poition.
   //
-  NoteTextMenu::NoteTextMenu(const Glib::RefPtr<Gtk::AccelGroup>& accel_group,
-                             const Glib::RefPtr<NoteBuffer> & buffer, UndoManager & undo_manager)
+  NoteTextMenu::NoteTextMenu(const Glib::RefPtr<NoteBuffer> & buffer, UndoManager & undo_manager)
     : Gtk::Menu()
     , m_buffer(buffer)
     , m_undo_manager(undo_manager)
@@ -1223,22 +1121,12 @@ namespace gnote {
     , m_decrease_font(_("Decrease Font Size"), true)
     {
       m_undo = manage(new Gtk::ImageMenuItem (Gtk::Stock::UNDO));
-//      m_undo->set_accel_group(accel_group);
       m_undo->signal_activate().connect(sigc::mem_fun(*this, &NoteTextMenu::undo_clicked));
-      m_undo->add_accelerator ("activate", accel_group,
-                               GDK_KEY_Z,
-                               Gdk::CONTROL_MASK,
-                               Gtk::ACCEL_VISIBLE);
       m_undo->show();
       append(*m_undo);
 
       m_redo = manage(new Gtk::ImageMenuItem (Gtk::Stock::REDO));
-//      m_redo->set_accel_group(accel_group);
       m_redo->signal_activate().connect(sigc::mem_fun(*this, &NoteTextMenu::redo_clicked));
-      m_redo->add_accelerator ("activate", accel_group,
-                               GDK_KEY_Z, (Gdk::CONTROL_MASK |
-                                       Gdk::SHIFT_MASK),
-                               Gtk::ACCEL_VISIBLE);
       m_redo->show();
       append(*m_redo);
 
@@ -1254,40 +1142,21 @@ namespace gnote {
       m_bold.set_data(tag_quark, const_cast<char*>("bold"));
       m_bold.signal_activate()
         .connect(sigc::bind(sigc::mem_fun(*this, &NoteTextMenu::font_style_clicked), &m_bold));
-      m_bold.add_accelerator ("activate", accel_group,
-                           GDK_KEY_B,
-                           Gdk::CONTROL_MASK,
-                           Gtk::ACCEL_VISIBLE);
 
       markup_label (m_italic);
       m_italic.set_data(tag_quark, const_cast<char*>("italic"));
       m_italic.signal_activate()
         .connect(sigc::bind(sigc::mem_fun(*this, &NoteTextMenu::font_style_clicked), &m_italic));
-      m_italic.add_accelerator ("activate",
-                             accel_group,
-                             GDK_KEY_I,
-                             Gdk::CONTROL_MASK,
-                             Gtk::ACCEL_VISIBLE);
 
       markup_label (m_strikeout);
       m_strikeout.set_data(tag_quark, const_cast<char*>("strikethrough"));
       m_strikeout.signal_activate()
         .connect(sigc::bind(sigc::mem_fun(*this, &NoteTextMenu::font_style_clicked), &m_strikeout));
-      m_strikeout.add_accelerator ("activate",
-                                accel_group,
-                                GDK_KEY_S,
-                                Gdk::CONTROL_MASK,
-                                Gtk::ACCEL_VISIBLE);
 
       markup_label (m_highlight);
       m_highlight.set_data(tag_quark, const_cast<char*>("highlight"));
       m_highlight.signal_activate()
         .connect(sigc::bind(sigc::mem_fun(*this, &NoteTextMenu::font_style_clicked), &m_highlight));
-      m_highlight.add_accelerator ("activate",
-                                accel_group,
-                                GDK_KEY_H,
-                                Gdk::CONTROL_MASK,
-                                Gtk::ACCEL_VISIBLE);
 
       Gtk::SeparatorMenuItem *spacer1 = manage(new Gtk::SeparatorMenuItem());
 
@@ -1316,19 +1185,9 @@ namespace gnote {
 
       m_hidden_no_size.hide();
 
-      m_increase_font.add_accelerator ("activate",
-                                       accel_group,
-                                       GDK_KEY_plus,
-                                       Gdk::CONTROL_MASK,
-                                       Gtk::ACCEL_VISIBLE);
       m_increase_font.signal_activate()
         .connect(sigc::mem_fun(*this, &NoteTextMenu::increase_font_clicked));
 
-      m_decrease_font.add_accelerator ("activate",
-                                       accel_group,
-                                       GDK_KEY_minus,
-                                       Gdk::CONTROL_MASK,
-                                       Gtk::ACCEL_VISIBLE);
       m_decrease_font.signal_activate()
         .connect(sigc::mem_fun(*this, &NoteTextMenu::decrease_font_clicked));
 
@@ -1337,20 +1196,10 @@ namespace gnote {
       m_bullets_clicked_cid = m_bullets.signal_activate()
         .connect(sigc::mem_fun(*this, &NoteTextMenu::toggle_bullets_clicked));
 
-//      m_increase_indent.set_accel_group(accel_group);
-      m_increase_indent.add_accelerator ("activate", accel_group,
-                                       GDK_KEY_Right,
-                                       Gdk::MOD1_MASK,
-                                       Gtk::ACCEL_VISIBLE);
       m_increase_indent.signal_activate()
         .connect(sigc::mem_fun(*this, &NoteTextMenu::increase_indent_clicked));
       m_increase_indent.show();
 
-//      m_decrease_indent.set_accel_group(accel_group);
-      m_decrease_indent.add_accelerator ("activate", accel_group,
-                                         GDK_KEY_Left,
-                                         Gdk::MOD1_MASK,
-                                         Gtk::ACCEL_VISIBLE);
       m_decrease_indent.signal_activate()
         .connect(sigc::mem_fun(*this, &NoteTextMenu::decrease_indent_clicked));
       m_decrease_indent.show();
@@ -1376,6 +1225,50 @@ namespace gnote {
       show_all ();
     }
 
+  void NoteTextMenu::set_accel_group(const Glib::RefPtr<Gtk::AccelGroup> & accel_group)
+  {
+    m_undo->add_accelerator("activate", accel_group,
+                            GDK_KEY_Z,
+                            Gdk::CONTROL_MASK,
+                            Gtk::ACCEL_VISIBLE);
+    m_redo->add_accelerator("activate", accel_group,
+                            GDK_KEY_Z,
+                            Gdk::CONTROL_MASK | Gdk::SHIFT_MASK,
+                            Gtk::ACCEL_VISIBLE);
+    m_bold.add_accelerator("activate", accel_group,
+                           GDK_KEY_B,
+                           Gdk::CONTROL_MASK,
+                           Gtk::ACCEL_VISIBLE);
+    m_italic.add_accelerator("activate", accel_group,
+                             GDK_KEY_I,
+                             Gdk::CONTROL_MASK,
+                             Gtk::ACCEL_VISIBLE);
+    m_strikeout.add_accelerator("activate", accel_group,
+                                GDK_KEY_S,
+                                Gdk::CONTROL_MASK,
+                                Gtk::ACCEL_VISIBLE);
+    m_highlight.add_accelerator("activate", accel_group,
+                                GDK_KEY_H,
+                                Gdk::CONTROL_MASK,
+                                Gtk::ACCEL_VISIBLE);
+    m_increase_font.add_accelerator("activate", accel_group,
+                                    GDK_KEY_plus,
+                                    Gdk::CONTROL_MASK,
+                                    Gtk::ACCEL_VISIBLE);
+    m_decrease_font.add_accelerator("activate", accel_group,
+                                    GDK_KEY_minus,
+                                    Gdk::CONTROL_MASK,
+                                    Gtk::ACCEL_VISIBLE);
+    m_increase_indent.add_accelerator("activate", accel_group,
+                                      GDK_KEY_Right,
+                                      Gdk::MOD1_MASK,
+                                      Gtk::ACCEL_VISIBLE);
+    m_decrease_indent.add_accelerator("activate", accel_group,
+                                      GDK_KEY_Left,
+                                      Gdk::MOD1_MASK,
+                                      Gtk::ACCEL_VISIBLE);
+  }
+
   void NoteTextMenu::on_show()
   {
     refresh_state();
