@@ -51,11 +51,10 @@ class NoteTextMenu
   : public Gtk::Menu
 {
 public:
-  NoteTextMenu(const Glib::RefPtr<Gtk::AccelGroup>&, 
-               const Glib::RefPtr<NoteBuffer> & buffer, UndoManager& undo_manager);
+  NoteTextMenu(const Glib::RefPtr<NoteBuffer> & buffer, UndoManager& undo_manager);
+  void set_accel_group(const Glib::RefPtr<Gtk::AccelGroup> &);
 
   static void markup_label (Gtk::MenuItem & item);
-  
 protected:
   virtual void on_show();
 
@@ -100,12 +99,32 @@ private:
 
 
 class NoteWindow 
-  : public utils::ForcedPresentWindow
+  : public Gtk::VBox
+  , public utils::EmbedableWidget
 {
 public:
   NoteWindow(Note &);
   ~NoteWindow();
 
+  virtual std::string get_name() const;
+  void set_name(const std::string & name)
+    {
+      m_name = name;
+    }
+  virtual void embed(utils::EmbedableWidgetHost *h);
+  virtual void foreground();
+  virtual void background();
+
+  void set_size(int width, int height)
+    {
+      m_width = width;
+      m_height = height;
+    }
+  void set_position(int x, int y)
+    {
+      m_x = x;
+      m_y = y;
+    }
   Gtk::TextView * editor() const
     {
       return m_editor;
@@ -134,21 +153,13 @@ public:
     {
       return *m_find_bar;
     }
-
-protected:
-  virtual bool on_delete_event(GdkEventAny *ev);
-  virtual void on_hide();
-  virtual void on_show();
 private:
   bool on_key_pressed(GdkEventKey*);
-  void close_window_handler();
-  void close_all_windows_handler();
   void on_delete_button_clicked();
   void on_selection_mark_set(const Gtk::TextIter&, const Glib::RefPtr<Gtk::TextMark>&);
   void update_link_button_sensitivity();
   void on_populate_popup(Gtk::Menu*);
   Gtk::Toolbar * make_toolbar();
-  void sync_item_selected();
   Gtk::Menu * make_plugin_menu();
   Gtk::Box * make_template_bar();
   void on_untemplate_button_click();
@@ -164,12 +175,17 @@ private:
   void find_bar_hidden();
   void link_button_clicked();
   void open_help_activate();
-  void create_new_note();
   void change_depth_right_handler();
   void change_depth_left_handler();
-  void search_button_clicked();
+  void add_accel_group(Gtk::Window &);
+  void remove_accel_group(Gtk::Window &);
 
   Note                        & m_note;
+  std::string                   m_name;
+  int                           m_height;
+  int                           m_width;
+  int                           m_x;
+  int                           m_y;
   Glib::RefPtr<Gtk::AccelGroup> m_accel_group;
   Gtk::Toolbar                 *m_toolbar;
   Gtk::ToolButton              *m_link_button;
@@ -183,6 +199,7 @@ private:
   Gtk::CheckButton             *m_save_size_check_button;
   Gtk::CheckButton             *m_save_selection_check_button;
   Gtk::CheckButton             *m_save_title_check_button;
+  Gtk::ImageMenuItem           *m_find_item;
 
   utils::GlobalKeybinder       *m_global_keys;
   utils::InterruptableTimeout  *m_mark_set_timeout;
