@@ -1007,7 +1007,7 @@ namespace gnote {
     Note::Ptr template_note = manager.get_or_create_template_note ();
 
     // Open the template note
-    template_note->get_window()->show ();
+    Gnote::obj().open_note(template_note);
   }
 
 
@@ -1251,7 +1251,7 @@ DBG_OUT("no addin");
 
       if(dialog_response == Gtk::RESPONSE_YES) {
         // TODO: Put this voodoo in a method somewhere
-        ActionManager::obj()["NoteSynchronizationAction"]->activate();
+        ActionManager::obj().get_app_action("sync-notes")->activate(Glib::VariantBase());
       }
     }
     else {
