@@ -26,6 +26,7 @@
 #include <sigc++/signal.h>
 
 #include <gdkmm/pixbuf.h>
+#include <gtkmm/applicationwindow.h>
 #include <gtkmm/dialog.h>
 #include <gtkmm/image.h>
 #include <gtkmm/menu.h>
@@ -207,18 +208,18 @@ namespace gnote {
     };
 
     class ForcedPresentWindow 
-      : public Gtk::Window
+      : public Gtk::ApplicationWindow
     {
     public:
       ForcedPresentWindow(const Glib::ustring & title)
-        : Gtk::Window()
+        : Gtk::ApplicationWindow()
         {
-          Gtk::Window::set_title(title);
+          Gtk::ApplicationWindow::set_title(title);
         }
 
       void present()
         {
-          ::tomboy_window_present_hardcore(GTK_WINDOW(Gtk::Window::gobj()));
+          ::tomboy_window_present_hardcore(GTK_WINDOW(gobj()));
         }
     };
 
@@ -242,6 +243,54 @@ namespace gnote {
       void release_button();        
     };
 
+    class EmbedableWidget;
+    class EmbedableWidgetHost
+    {
+    public:
+      virtual void embed_widget(EmbedableWidget &) = 0;
+      virtual void unembed_widget(EmbedableWidget &) = 0;
+      virtual void foreground_embeded(EmbedableWidget &) = 0;
+      virtual void background_embeded(EmbedableWidget &) = 0;
+      virtual bool running() = 0;
+    };
+
+    class EmbedableWidget
+    {
+    public:
+      EmbedableWidget() : m_host(NULL) {}
+      virtual std::string get_name() const = 0;
+      virtual void embed(EmbedableWidgetHost *h)
+        {
+          m_host = h;
+          signal_embeded();
+        }
+      virtual void unembed()
+        {
+          m_host = NULL;
+          signal_unembeded();
+        }
+      virtual void foreground()
+        {
+          signal_foregrounded();
+        }
+      virtual void background()
+        {
+          signal_backgrounded();
+        }
+      EmbedableWidgetHost *host() const
+        {
+          return m_host;
+        }
+
+      sigc::signal<void, const std::string &> signal_name_changed;
+      sigc::signal<void> signal_embeded;
+      sigc::signal<void> signal_unembeded;
+      sigc::signal<void> signal_foregrounded;
+      sigc::signal<void> signal_backgrounded;
+    private:
+      EmbedableWidgetHost *m_host;
+    };
+
 
   }
 }
